<?php

namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DataSource\Manager;
use Teknisa\MAN\Services\ConversaoCodFilialService;
use Zeedhi\Framework\DTO\Response\Message;
use \Exception as Exception;

class ConversaoCodFilial extends CrudWithParameterBag {
    
    const dataSource               = 'vnd32000_conversaoCodFilial';
    protected $parameterBagColumns = array('NRORG');
    protected $crudService;
    
              
    public function __construct (ParameterBag $parameterBag, Manager $dataSourceManager, $crudService, ConversaoCodFilialService $conversaoCodFilialService) {
        $this->parameterBag = $parameterBag;
        $this->dataSourceManager = $dataSourceManager;
        $this->crudService = $crudService;
        $this->conversaoCodFilialService = $conversaoCodFilialService;
    }
    
    public function find(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName      = self::dataSource;
        $this->parameterBagColumns = $this->parameterBagColumns;
        parent::find($request, $response);     
    }
   
    public function save(DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = self::dataSource;
        $dataSet = $request->getDataSet();
        $rows = $dataSet->getRows()[0];
        $isRegistered = false;
        
        if($rows['__is_new']) {
            $retornoCodigos = $this->conversaoCodFilialService->findCodigos($rows['NRORG']);
            
            foreach($retornoCodigos as $valor) {
                if(($valor['CDCODEXTINI'] < $rows['CDCODEXTINI'] && $valor['CDCODEXTFIN'] > $rows['CDCODEXTFIN']) || $valor['CDCODEXTINI'] == $rows['CDCODEXTINI'] || $valor['CDCODEXTFIN'] == $rows['CDCODEXTFIN'] || $valor['CDCODEXTFIN'] == $rows['CDCODEXTINI'] || $valor['CDCODEXTINI'] == $rows['CDCODEXTFIN']) {
                    $isRegistered = true;
                }
            }
        } else {
            $retornoCodigos = $this->conversaoCodFilialService->findOutrosCodigos($rows['NRORG'], $rows['LETRAREF']);
            
            foreach($retornoCodigos as $valor) {
                if(($valor['CDCODEXTINI'] < $rows['CDCODEXTINI'] && $valor['CDCODEXTFIN'] > $rows['CDCODEXTFIN']) || $valor['CDCODEXTINI'] == $rows['CDCODEXTINI'] || $valor['CDCODEXTFIN'] == $rows['CDCODEXTFIN'] || $valor['CDCODEXTFIN'] == $rows['CDCODEXTINI'] || $valor['CDCODEXTINI'] == $rows['CDCODEXTFIN']) {
                    $isRegistered = true;
                }
            }
            
            if(!$isRegistered) {
                try {
                    $this->conversaoCodFilialService->updateCampos($rows['CDCODEXTINI'], $rows['CDCODEXTFIN'], $rows['CDLETRACORR'], $rows['NRORG'], $rows['LETRAREF']);
                    $response->addMessage(new Message("Registro atualizado!"));
                } catch (Exception $e) {
                    $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
                }
            } else {
                $response->setError(new DTO\Response\Error("Unidade com cadastro já realizado! Favor selecionar um intervalo de Unidades disponível.", '01'));
            }
        }
        
        if (!$isRegistered) {
            try {
                $dataSet = $request->getDataSet();
                $dataSet->setDataSourceName($this->dataSourceName);
                $persistedRows = $this->dataSourceManager->persist($dataSet);
                $response->addDataSet(new DataSet($this->dataSourceName, $persistedRows));
                $nrPersistedRows = count($persistedRows);
            } catch (Exception $e) {
                $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
            }    
        } else {
            $response->setError(new DTO\Response\Error("Unidade com cadastro já realizado! Favor selecionar um intervalo de Unidades disponível.", '01'));
        }
    }
    
    public function delete(DTO\Request $request, DTO\Response $response) {
        $request = $this->crudService->buildDataSetIfRow($request, $this->dataSourceName);
        $this->dataSourceName = self::dataSource;
        
        try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $deletedRows = $this->dataSourceManager->delete($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $deletedRows));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
}
