<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use \Zeedhi\Framework\Util\Functions;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Request;
use Teknisa\MAN\Services;
use Teknisa\MAN\Utilities;
use \Exception as Exception;

class RentProdResOpCadController {
    protected $RentProdResOpCadService;
    protected $utilService;

    public function __construct(Services\RentProdResOpCadService $RentProdResOpCadService, Utilities\Util $utilService) {
        $this->RentProdResOpCadService = $RentProdResOpCadService;
        $this->utilService = $utilService;
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        $result = [];
        $conditions = $this->utilService->getParamsAsArray($request);
        
        if(isset($conditions['CDFILIAL']) && !empty($conditions['CDFILIAL'])) {
            $result = $this->RentProdResOpCadService->find($conditions);
            
            foreach($result as $index => $row) {
                $result[$index]['NMMES'] = "".$row['CDMES'];
            }
            
            $response->addDataSet(new DataSet("rentProdResOpCad", $result));
        }

        return $result;
    }
    
    public function save(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $rows = $request->getDataSet()->getRows()[0];
            
            if($rows['__is_new']) {
                // validação 
                $validacao = $this->RentProdResOpCadService->validacao($rows);
                
                if(empty($validacao)) {
                    // INSERT
                    $this->RentProdResOpCadService->save($rows);
                    
                    $response->addNotification(new Notification("Dados inseridos com sucesso!", Notification::TYPE_SUCCESS));
                } else {
                    throw new Exception("Cadastro já existe!");
                }
            } else {
                // UPDATE
                $this->RentProdResOpCadService->update($rows);
                
                $response->addNotification(new Notification("Dados atualizados com sucesso!", Notification::TYPE_SUCCESS));
            }
        } catch (Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function saveInline(DTO\Request $request, DTO\Response $response) {
        try {
            $rows = $request->getRow();
            
            foreach($rows as $row) {
                $this->RentProdResOpCadService->update($row);
            }
            
            $response->addNotification(new Notification("Dados atualizados com sucesso!", Notification::TYPE_SUCCESS));
        } catch (Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}